<?php
namespace Ak\Orm\Filter;

use Ak\Orm\Service\EntityManagerAwareInterface;
use Ak\Orm\Service\EntityManagerAwareTrait;
use Doctrine\ORM\EntityManagerInterface;
use Zend\Filter\AbstractFilter;
use Zend\Filter\Exception;

/**
 * Pretvara id-eve entiteta u entitete.
 *
 * @package Ak\Orm\Filter
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/orm
 */
final class Entity extends AbstractFilter implements EntityManagerAwareInterface
{
    use EntityManagerAwareTrait;

    /** @var string */
    private $entityClass;

    /**
     * EntityFilter constructor.
     *
     * @param string|array $entityClassOrOptions
     */
    public function __construct($entityClassOrOptions = null)
    {
        if (is_string($entityClassOrOptions)) {
            $this->setEntityClass($entityClassOrOptions);
        } else if (is_array($entityClassOrOptions)) {
            $this->setOptions($entityClassOrOptions);
        }
    }

    /**
     * @return string
     */
    public function getEntityClass(): ?string
    {
        return $this->entityClass;
    }

    /**
     * @param string $entityClass
     */
    public function setEntityClass(string $entityClass): void
    {
        $this->entityClass = $entityClass;
    }

    /**
     * Returns the result of filtering $value
     *
     * @param  mixed $value
     * @throws Exception\RuntimeException If filtering $value is impossible
     * @return mixed
     */
    public function filter($value)
    {
        if (!is_numeric($value)) {
            throw new Exception\RuntimeException(sprintf(
                "Data vrijednost mora biti broj, a dobio sam %s.", $value
            ));
        }

        $em = $this->getEntityManager();
        $entityClass = $this->getEntityClass();

        if (!$em instanceof EntityManagerInterface) {
            throw new Exception\RuntimeException('Nije postavljen entity manager.');
        }

        if (empty($entityClass)) {
            throw new Exception\RuntimeException('Nije postavljena klasa entiteta.');
        }

        return $em->find($entityClass, $value);
    }
}